//Copyright (c) 2006-2007 Microsoft Corporation.  All rights reserved
using System;
using System.ComponentModel;
using System.Collections;
using System.Configuration.Install;
using System.Diagnostics;

namespace InvocationFramework.Common.Instrumentation
{
    /// <summary>
    /// Clase installer para agregar los performance counter utilizados por la aplicacin
    /// </summary>
    /// <author>Rodolfo Finochietti 02/01/2006 12.55</author>
    [RunInstaller( true )]
    public class InstrumentationInstaller : Installer
    {
        #region Constants

        private const string General = "Services Invocation Framework";

        #endregion

        #region Private Vars

        private System.ComponentModel.Container components;

        #endregion

        #region Constructors

        /// <summary>
        /// Constructor
        /// </summary>
        public InstrumentationInstaller( )
        {
            InitializeComponent( );
        }

        #endregion

        #region Events

        /// <summary>
        /// </summary>
        /// <param name="stateSaver"></param>
        public override void Install( IDictionary stateSaver )
        {
            base.Install( stateSaver );

            if ( !PerformanceCounterCategory.Exists( General ) )
            {
                CounterCreationDataCollection counters = new CounterCreationDataCollection( );

                CounterCreationData invocationsTotal = new CounterCreationData( );
                invocationsTotal.CounterName = "# of invocations";
                invocationsTotal.CounterHelp = "Total number of invocations";
                invocationsTotal.CounterType = PerformanceCounterType.NumberOfItems32;
                counters.Add( invocationsTotal );

                CounterCreationData invocationsPerSecond = new CounterCreationData( );
                invocationsPerSecond.CounterName = "# of invocations per sec";
                invocationsPerSecond.CounterHelp = "Number of invocations per second";
                invocationsPerSecond.CounterType = PerformanceCounterType.RateOfCountsPerSecond32;
                counters.Add( invocationsPerSecond );

                CounterCreationData averageInvocationTotal = new CounterCreationData( );
                averageInvocationTotal.CounterName = "average time per invocation execution";
                averageInvocationTotal.CounterHelp = "Average duration per invocation execution";
                averageInvocationTotal.CounterType = PerformanceCounterType.AverageTimer32;
                counters.Add( averageInvocationTotal );

                CounterCreationData averageInvocationTotalBase = new CounterCreationData( );
                averageInvocationTotalBase.CounterName = "average time per invocation execution base";
                averageInvocationTotalBase.CounterHelp = "Average duration per invocation execution base";
                averageInvocationTotalBase.CounterType = PerformanceCounterType.AverageBase;
                counters.Add( averageInvocationTotalBase );

                //Creo la categoria con los contadores previamente mencioandos
                PerformanceCounterCategory.Create( General, "Services Invocation Framework General Counters", PerformanceCounterCategoryType.MultiInstance, counters );
            }
        }

        /// <summary>
        /// </summary>
        /// <param name="savedState"></param>
        public override void Uninstall( IDictionary savedState )
        {
            base.Uninstall( savedState );

            if ( PerformanceCounterCategory.Exists( General ) )
                PerformanceCounterCategory.Delete( General );
        }

        #endregion

        #region Private Methods

        #region Component Designer generated code

        private void InitializeComponent( )
        {
            components = new System.ComponentModel.Container( );
        }

        #endregion

        /// <summary>
        /// </summary>
        /// <param name="disposing"></param>
        protected override void Dispose( bool disposing )
        {
            try
            {
                if ( disposing )
                {
                    if ( components != null )
                        components.Dispose( );
                }
            }
            finally
            {
                base.Dispose( disposing );
            }
        }

        #endregion
    }
}