//Copyright (c) 2006-2007 Microsoft Corporation.  All rights reserved
using System;
using System.Data;

using Microsoft.VisualStudio.TestTools.UnitTesting;

using InvocationFramework.Core.ProxyFactory;

using ServicedComponentAdapterTest.Contracts;

namespace ServicedComponentAdapterTest.UnitTests
{
    [TestClass]
    public class COMPlusInvocations
    {
        #region Private Vars

        private ITestContract _service;

        #endregion

        #region Constructors

        public COMPlusInvocations( ) { }

        #endregion

        #region Support Methods

        [TestInitialize( )]
        public void Initialize( )
        {
            if ( _service == null )
            {
                ServiceProxyBuilder<ITestContract> factory = new ServiceProxyBuilder<ITestContract>( "ServicedComponentAdapterTest.Services.TestService" );
                _service = factory.BuildProxy( );
            }
        }

        #endregion

        #region Test Methods

        [TestMethod]
        [Owner( "Rodolfo Finochietti" )]
        public void InvokeCOMPlusStringOperation( )
        {
            Assert.AreEqual( "UnitTest", _service.GetSameString( "UnitTest" ) );
        }

        [TestMethod]
        [Owner( "Rodolfo Finochietti" )]
        public void InvokeCOMPlusDataSetOperation( )
        {
            DataSet ds = _service.GetDataSet( );

            Assert.AreEqual( "UnitTestDataSet", ds.DataSetName );
            Assert.AreEqual( 1, ds.Tables.Count );
            Assert.AreEqual( "UnitTestTable", ds.Tables[ 0 ].TableName );
            Assert.AreEqual( 2, ds.Tables[ 0 ].Columns.Count );
            Assert.AreEqual( "Id", ds.Tables[ 0 ].Columns[ 0 ].ColumnName );
            Assert.AreEqual( "Description", ds.Tables[ 0 ].Columns[ 1 ].ColumnName );
            Assert.AreEqual( 8, ds.Tables[ 0 ].Rows.Count );
        }

        [TestMethod]
        [ExpectedException( typeof( ApplicationException ) )]
        [Owner( "Rodolfo Finochietti" )]
        public void InvokeCOMPlusExceptionOperation( )
        {
            _service.ExceptionOperation( );
        }

        #endregion
    }
}