//Copyright (c) 2006-2007 Microsoft Corporation.  All rights reserved
using System;
using System.Web.Services.Protocols;

using Microsoft.VisualStudio.TestTools.UnitTesting;

using InvocationFramework.Core.ProxyFactory;

using WebServiceAdapterTests.Common.Contracts;

namespace WebServiceAdapterTests.UnitTests
{
    [TestClass]
    public class BasicInvocations
    {
        #region Private Vars

        private IBasicService _service;

        #endregion

        #region Constructors

        public BasicInvocations( ) { }

        #endregion

        #region Support Methods

        [TestInitialize( )]
        public void Initialize( )
        {
            if ( _service == null )
            {
                ServiceProxyBuilder<IBasicService> factory = new ServiceProxyBuilder<IBasicService>( "BasicService" );
                _service = factory.BuildProxy( );
            }
        }

        #endregion

        #region Test Methods

        [TestMethod]
        [Owner( "Rodolfo Finochietti" )]
        public void InvokeSimpleStringOperation( )
        {
            Assert.AreEqual( "UnitTest", _service.GetSameString( "UnitTest" ) );
        }

        [TestMethod]
        [Owner( "Rodolfo Finochietti" )]
        public void InvokeSimpleInt32Operation( )
        {
            Assert.AreEqual( 10, _service.GetSameInt32( 10 ) );
        }

        [TestMethod]
        [Owner( "Rodolfo Finochietti" )]
        public void InvokeVoidOperation( )
        {
            _service.VoidOperation( );
        }

        [TestMethod]
        [ExpectedException( typeof( SoapException ) )]
        [Owner( "Rodolfo Finochietti" )]
        public void InvokeExceptionOperation( )
        {
            _service.ExceptionOperation( );
        }

        [TestMethod]
        [Owner( "Rodolfo Finochietti" )]
        public void InvokeNoNameService( )
        {
            ServiceProxyBuilder<IBasicService> factory = new ServiceProxyBuilder<IBasicService>( );
            IBasicService service = factory.BuildProxy( );

            Assert.AreEqual( "UnitTest", service.GetSameString( "UnitTest" ) );
        }

        #endregion
   }
}