//Copyright (c) 2006-2007 Microsoft Corporation.  All rights reserved
using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;

using Microsoft.VisualStudio.TestTools.UnitTesting;

using InvocationFramework.Core.ProxyFactory;

using WebServiceAdapterTests.Common.Contracts;
using WebServiceAdapterTests.Common.DataTransferObjects;

namespace WebServiceAdapterTests.UnitTests
{
    [TestClass]
    public class DTOInvocations
    {
        #region Private Vars

        private IDTOService _service;

        #endregion

        #region Constructors

        public DTOInvocations( ) { }

        #endregion

        #region Support Methods

        [TestInitialize( )]
        public void Initialize( )
        {
            if ( _service == null )
            {
                ServiceProxyBuilder<IDTOService> factory = new ServiceProxyBuilder<IDTOService>( "DTOService" );
                _service = factory.BuildProxy( );
            }
        }

        #endregion

        #region TestMethods

        [TestMethod]
        [Owner( "Rodolfo Finochietti" )]
        public void InvokeBasicDTOOperation( )
        {
            BasicDto requestDTO = new BasicDto( );
            requestDTO.StringProperty = "UnitTest";
            requestDTO.Int32Property = 100;
            requestDTO.StringArrayProperty = new string[] { "1", "2" };

            BasicDto responseDTO = _service.GetBasicDto( requestDTO );

            Assert.AreEqual( requestDTO.StringProperty, responseDTO.StringProperty );

            Assert.AreEqual( requestDTO.Int32Property, responseDTO.Int32Property );

            Assert.AreEqual( requestDTO.StringArrayProperty.GetUpperBound( 0 ), responseDTO.StringArrayProperty.GetUpperBound( 0 ) );
            Assert.AreEqual( requestDTO.StringArrayProperty[ 0 ], responseDTO.StringArrayProperty[ 0 ] );
            Assert.AreEqual( requestDTO.StringArrayProperty[ 1 ], responseDTO.StringArrayProperty[ 1 ] );
        }

        [TestMethod]
        [Owner( "Rodolfo Finochietti" )]
        public void InvokeDataDTOOperation( )
        {
            DataTable table = new DataTable( "UnitTestTable" );

            table.Columns.Add( "Id", typeof( int ) );
            table.Columns.Add( "Description", typeof( string ) );

            table.Rows.Add( 1, "Row 1" );
            table.Rows.Add( 2, "Row 2" );
            table.Rows.Add( 3, "Row 3" );

            DataDto requestDTO = new DataDto( );
            requestDTO.DataSetProperty = new DataSet( "UnitTestDataSet" );
            requestDTO.DataSetProperty.Tables.Add( table );

            DataDto responseDTO = _service.GetDataDto( requestDTO );

            Assert.AreEqual( "UnitTestDataSet", responseDTO.DataSetProperty.DataSetName );
            Assert.AreEqual( 1, responseDTO.DataSetProperty.Tables.Count );
            Assert.AreEqual( "UnitTestTable", responseDTO.DataSetProperty.Tables[ 0 ].TableName );
            Assert.AreEqual( 2, responseDTO.DataSetProperty.Tables[ 0 ].Columns.Count );
            Assert.AreEqual( "Id", responseDTO.DataSetProperty.Tables[ 0 ].Columns[ 0 ].ColumnName );
            Assert.AreEqual( "Description", responseDTO.DataSetProperty.Tables[ 0 ].Columns[ 1 ].ColumnName );
            Assert.AreEqual( table.Rows.Count, responseDTO.DataSetProperty.Tables[ 0 ].Rows.Count );
        }

        [TestMethod]
        [Owner( "Rodolfo Finochietti" )]
        public void InvokeCollectionDTOOperation( )
        {
            CollectionDto requestDTO = new CollectionDto( );
            requestDTO.ArrayListProperty = new ArrayList( );
            requestDTO.ArrayListProperty.Add( "1" );
            requestDTO.ArrayListProperty.Add( "2" );
            requestDTO.GenericListProperty = new List<int>( );
            requestDTO.GenericListProperty.Add( 1 );
            requestDTO.GenericListProperty.Add( 2 );

            CollectionDto responseDTO = _service.GetCollectionDto( requestDTO );

            Assert.AreEqual( requestDTO.ArrayListProperty.Count, responseDTO.ArrayListProperty.Count );
            Assert.IsTrue( responseDTO.ArrayListProperty.Contains( "1" ) );
            Assert.IsTrue( responseDTO.ArrayListProperty.Contains( "2" ) );

            Assert.AreEqual( requestDTO.GenericListProperty.Count, responseDTO.GenericListProperty.Count );
            Assert.IsTrue( responseDTO.GenericListProperty.Contains( 1 ) );
            Assert.IsTrue( responseDTO.GenericListProperty.Contains( 2 ) );
        }

        [TestMethod]
        [Owner( "Rodolfo Finochietti" )]
        public void InvokeComplexDTOOperation( )
        {
            ComplexDto responseDTO = _service.GetComplexDto( );

            Assert.AreEqual( "One", responseDTO.Basic.StringProperty );
            Assert.AreEqual( 1, responseDTO.Basic.Int32Property );
            Assert.AreEqual( 2, responseDTO.Basic.StringArrayProperty.GetUpperBound( 0 ) );
            Assert.AreEqual( "1", responseDTO.Basic.StringArrayProperty[ 0 ] );
            Assert.AreEqual( "2", responseDTO.Basic.StringArrayProperty[ 1 ] );

            Assert.AreEqual( 2, responseDTO.Collection.ArrayListProperty.Count );
            Assert.IsTrue( responseDTO.Collection.ArrayListProperty.Contains( "1" ) );
            Assert.IsTrue( responseDTO.Collection.ArrayListProperty.Contains( "2" ) );
            Assert.AreEqual( 2, responseDTO.Collection.GenericListProperty.Count );
            Assert.IsTrue( responseDTO.Collection.GenericListProperty.Contains( 1 ) );
            Assert.IsTrue( responseDTO.Collection.GenericListProperty.Contains( 2 ) );

            Assert.AreEqual( "UnitTestDataSet", responseDTO.Data.DataSetProperty.DataSetName );
            Assert.AreEqual( 1, responseDTO.Data.DataSetProperty.Tables.Count );
            Assert.AreEqual( "UnitTestTable", responseDTO.Data.DataSetProperty.Tables[ 0 ].TableName );
            Assert.AreEqual( 2, responseDTO.Data.DataSetProperty.Tables[ 0 ].Columns.Count );
            Assert.AreEqual( "Id", responseDTO.Data.DataSetProperty.Tables[ 0 ].Columns[ 0 ].ColumnName );
            Assert.AreEqual( "Description", responseDTO.Data.DataSetProperty.Tables[ 0 ].Columns[ 1 ].ColumnName );
            Assert.AreEqual( 5, responseDTO.Data.DataSetProperty.Tables[ 0 ].Rows.Count );
        }

        #endregion
    }
}
