//Copyright (c) 2006-2007 Microsoft Corporation.  All rights reserved
using System;
using System.Data;

using Microsoft.VisualStudio.TestTools.UnitTesting;

using InvocationFramework.Core.ProxyFactory;

using WebServiceAdapterTests.Common.Contracts;
using WebServiceAdapterTests.Common.DataTransferObjects;

namespace WebServiceAdapterTests.UnitTests
{
    [TestClass]
    public class DataInvocations
    {
        #region Private Vars

        private IDataService _service;

        #endregion

        #region Constructors

        public DataInvocations( ) { }

        #endregion

        #region Support Methods

        [TestInitialize( )]
        public void Initialize( )
        {
            if ( _service == null )
            {
                ServiceProxyBuilder<IDataService> factory = new ServiceProxyBuilder<IDataService>( "DataService" );
                _service = factory.BuildProxy( );
            }
        }
    
        #endregion

        #region Test Methods

        [TestMethod]
        [Owner( "Rodolfo Finochietti" )]
        public void InvokeDataSetOperation( )
        {
            DataSet ds = _service.GetDataSet( );

            Assert.AreEqual( "UnitTestDataSet", ds.DataSetName );
            Assert.AreEqual( 1, ds.Tables.Count );
            Assert.AreEqual( "UnitTestTable", ds.Tables[ 0 ].TableName );
            Assert.AreEqual( 2, ds.Tables[ 0 ].Columns.Count );
            Assert.AreEqual( "Id", ds.Tables[ 0 ].Columns[ 0 ].ColumnName );
            Assert.AreEqual( "Description", ds.Tables[ 0 ].Columns[ 1 ].ColumnName );
            Assert.AreEqual( 8, ds.Tables[ 0 ].Rows.Count );
        }

        [TestMethod]
        [Owner( "Rodolfo Finochietti" )]
        public void InvokeGetTypedDSOperation( )
        {
            TypedDS.CurrencyDataTable table = new TypedDS.CurrencyDataTable( );
            TypedDS.CurrencyDataTable result = _service.GetTypedDS( table );

            Assert.AreEqual( 2, result.Rows.Count );
            Assert.AreEqual( "1", result.Rows[ 0 ][ 0 ].ToString( ) );
            Assert.AreEqual( "Row 1", result.Rows[ 0 ][ 1 ].ToString( ) );
            Assert.AreEqual( "2", result.Rows[ 1 ][ 0 ].ToString( ) );
            Assert.AreEqual( "Row 2", result.Rows[ 1 ][ 1 ].ToString( ) );
        }

        #endregion
    }
}